@@ -251,6 +251,7 @@ typedef struct
 } sStatments;
 
 sStatments* sStatments_new();
+sStatments* sStatments_new2(sStatments* statments);
 void sStatments_delete(sStatments* self);
 BOOL saphire_compile2(char* cmdline, char* fname, sStatments* statments);
     // cmdlineのソースをコンパイルして結果をstatmentsに格納する。
@@ -315,5 +316,14 @@ extern vector_obj* gSaphireVarCompletions;
 void saphire_make_var_completion_list();          
 // 変数補完候補を更新
 
+typedef struct {
+    sStatments* mStatments;
+    vector_obj* mArgs;
+} sBlock;
+// ブロック
+
+void err_msg(char* msg, char* sname, int line);
+    // エラーメッセージを設定
+
 #endif
 
