@@ -56,8 +56,8 @@ char* gStatmentKindStrs[kCommand+1] = {
     "load",
     "while", "break", "exit", "if", "split", 
     "x", "join", "select", "lines", "rows",
-    "ary_new", "ary_add", "ary_erase", "ary_clear", 
-    "hash_new", "hash_add", "hash_erase", "hash_clear",
+    "ary_new", "ary_add", "ary_erase", 
+    "hash_new", "hash_add", "hash_erase",
     "match", "scan", "erase", "sub",  
     "read",
     "cd", 
@@ -128,8 +128,6 @@ BOOL gJobControl;
     // ジョブのコントロールをするかどうか
 vector_obj* gJobs;
     // ジョブ sJob*の配列
-list_obj* gSigChldRecieved;  
-    // SIGCHLDを受け取ったPID,STATUSのリスト
 
 char* gMagicNumber = "SAPHIRE_OBJ";  
     // オブジェクトファイルのマジックナンバー
@@ -176,13 +174,6 @@ hash_obj* gFWriteBuffers;
     // キーはファイルディスクリプタ番号の文字列
     // 内容は書き込んだファイルの内容
 
-#if defined(__LINUX__) || defined(__DARWIN__) || defined(__FREEBSD__)
-int gMsgId;
-    // メッセージキュー
-    // fork後に親プロセスと子プロセスの通信に使われる
-    // (プロセスグループが変わった場合に)
-#endif
-
 hash_obj* gInnerCommands;
     // 外部登録の内部コマンド
     // saphireを組み込んだアプリケーションで
@@ -198,9 +189,6 @@ vector_obj* gPSubTmpFiles;
     // プロセス置換で使われる一時ファイルを行末で消すために
     // 保存しておく配列
 
-hash_obj* gMatchRestart;
-    // matchで使う変数
-
 vector_obj* gStackTraceFName;
 vector_obj* gStackTraceLineNum;
     // スタックの記録
@@ -703,7 +691,6 @@ void sig_user()
 
 void sigchld_action(int signum, siginfo_t* info, void* ctx)
 {
-    list_push_back(gSigChldRecieved, (void*)info->si_pid);
 }
 
 void sigchld_block(int block)
@@ -795,7 +782,7 @@ void saphire_set_signal()
     struct sigaction sa;
     memset(&sa, 0, sizeof(sa));
     sa.sa_sigaction = sigchld_action;
-    sa.sa_flags = SA_SIGINFO;
+    sa.sa_flags = SA_RESTART|SA_SIGINFO;
     if(sigaction(SIGCHLD, &sa, NULL) < 0) {
         perror("sigaction1");
         exit(1);
@@ -878,7 +865,7 @@ void saphire_set_signal_optc()
     struct sigaction sa;
     memset(&sa, 0, sizeof(sa));
     sa.sa_sigaction = sigchld_action;
-    sa.sa_flags = SA_SIGINFO;
+    sa.sa_flags = SA_SIGINFO|SA_RESTART;
     if(sigaction(SIGCHLD, &sa, NULL) < 0) {
         perror("sigaction1");
         exit(1);
@@ -994,6 +981,15 @@ sRedirect* sRedirect_new(enum eRedirect type, int fd, char* fname)
     return self;
 }
 
+sRedirect* sRedirect_new2(sRedirect* redirect)
+{
+    sRedirect* self = (sRedirect*)MALLOC(sizeof(sRedirect));
+    self->mType = redirect->mType;
+    self->mFd = redirect->mFd;
+    self->mFName = STRING_NEW(string_c_str(redirect->mFName));
+    return self;
+}
+
 void sRedirect_delete(sRedirect* self)
 {
     string_delete(self->mFName);
@@ -1097,6 +1093,26 @@ sVar* sVar_new(MANAGED string_obj* name, MANAGED string_obj* index, MANAGED stri
     return self;
 }
 
+sVar* sVar_new2(sVar* var)
+{
+    sVar* self = MALLOC(sizeof(sVar));
+    self->mName = STRING_NEW(string_c_str(var->mName));
+    if(var->mIndex) {
+        self->mIndex = STRING_NEW(string_c_str(var->mIndex));
+    }
+    else {
+        self->mIndex = NULL;
+    }
+    if(var->mIndex2) {
+        self->mIndex2 = STRING_NEW(string_c_str(var->mIndex2));
+    }
+    else {
+        self->mIndex2 = NULL;
+    }
+    self->mDelimiter = STRING_NEW(string_c_str(var->mDelimiter));
+    return self;
+}
+
 void sVar_delete(sVar* self)
 {
     string_delete(self->mName);
@@ -1236,6 +1252,40 @@ sArg* sArg_new(int kind, void* body)
     return self;
 }
 
+sArg* sArg_new2(sArg* arg)
+{
+    sArg* self = MALLOC(sizeof(sArg));
+
+    self->mKind = arg->mKind;
+    switch(self->mKind) {
+    case 0:
+        self->mBody = STRING_NEW(string_c_str(arg->mBody));
+        break;
+
+    case 1:
+        self->mBody = sVar_new2(arg->mBody);
+        break;
+
+    case 2:
+        self->mBody = sStatments_new2(arg->mBody);
+        break;
+
+    case 3:
+        self->mBody = sVar_new2(arg->mBody);
+        break;
+
+    case 4:
+        self->mBody = sStatments_new2(arg->mBody);
+        break;
+
+    case 5:
+        self->mBody = sBlock_new2(arg->mBody);
+        break;
+    }
+
+    return self;
+}
+
 void sArg_delete(sArg* self)
 {
     switch(self->mKind) {
@@ -1433,6 +1483,15 @@ sSubshell* sSubshell_new(MANAGED sStatments* contents)
     return self;
 }
 
+sSubshell* sSubshell_new2(sSubshell* subshell)
+{
+    sSubshell* self = MALLOC(sizeof(sSubshell));
+
+    self->contents = sStatments_new2(subshell->contents);
+
+    return self;
+}
+
 void sSubshell_delete(sSubshell* self)
 {
     sStatments_delete(self->contents);
@@ -1472,6 +1531,16 @@ sTry* sTry_new(MANAGED sStatments* loops, MANAGED sStatments* loops2)
     return self;
 }
 
+sTry* sTry_new2(sTry* try_)
+{
+    sTry* self = MALLOC(sizeof(sTry));
+
+    self->loops = sStatments_new2(try_->loops);
+    self->loops2 = sStatments_new2(try_->loops2);
+
+    return self;
+}
+
 void sTry_delete(sTry* self)
 {
     sStatments_delete(self->loops);
@@ -1515,6 +1584,16 @@ sWhile* sWhile_new(MANAGED sStatments* joukensiki, MANAGED sStatments* contents)
     return self;
 }
 
+sWhile* sWhile_new2(sWhile* while_)
+{
+    sWhile* self = MALLOC(sizeof(sWhile));
+
+    self->joukensiki = sStatments_new2(while_->joukensiki);
+    self->contents = sStatments_new2(while_->contents);
+
+    return self;
+}
+
 void sWhile_delete(sWhile* self)
 {
     sStatments_delete(self->joukensiki);
@@ -1558,6 +1637,24 @@ sIf* sIf_new(MANAGED vector_obj* joukensiki_list, MANAGED vector_obj* contents_l
     return self;
 }
 
+sIf* sIf_new2(sIf* if_)
+{
+    sIf* self = MALLOC(sizeof(sIf));
+    
+    self->joukensiki_list = VECTOR_NEW(10);
+    self->contents_list = VECTOR_NEW(10);
+
+    int i;
+    for(i=0; i<vector_size(if_->joukensiki_list); i++) {
+        vector_add(self->joukensiki_list, sStatments_new2(vector_item(if_->joukensiki_list, i)));
+    }
+    for(i=0; i<vector_size(if_->contents_list); i++) {
+        vector_add(self->contents_list, sStatments_new2(vector_item(if_->contents_list, i)));
+    }
+
+    return self;
+}
+
 void sIf_delete(sIf* self)
 {
     int i;
@@ -1682,6 +1779,53 @@ sCommand* sCommand_new()
 
 #endif
 
+sCommand* sCommand_new2(sCommand* command)
+{
+    sCommand* self = MALLOC(sizeof(sCommand));
+
+    self->mKind = command->mKind;
+    self->mArgTmps = VECTOR_NEW(50);
+
+    self->mArgs = VECTOR_NEW(50);
+    int i;
+    for(i=0; i<vector_size(command->mArgs); i++) {
+        vector_add(self->mArgs, sArg_new2(vector_item(command->mArgs, i)));
+    }
+    self->mRedirects = VECTOR_NEW(10);
+    for(i=0; i<vector_size(command->mRedirects); i++) {
+        vector_add(self->mRedirects, sRedirect_new2(vector_item(command->mRedirects, i)));
+    }
+    /// mExtra ///
+    switch(command->mKind) {
+    case kIf:
+        self->mExtra = sIf_new2(command->mExtra);
+        break;
+        
+    case kWhile:
+        self->mExtra = sWhile_new2(command->mExtra);
+        break;
+        
+    case kTry:
+        self->mExtra = sTry_new2(command->mExtra);
+        break;
+
+    case kSubshell:
+        self->mExtra = sSubshell_new2(command->mExtra);
+        break;
+
+    case kDef:
+        self->mExtra = sFunction_new2(command->mExtra);
+        break;
+
+    default:
+        self->mExtra = NULL;
+        break;
+    }
+
+    return self;
+}
+
+
 void sCommand_view(sCommand* self)
 {
     printf("self->mKind %s\n", gStatmentKindStrs[self->mKind]);
@@ -1943,6 +2087,35 @@ sStatment* sStatment_new()
 
 #endif
 
+sStatment* sStatment_new2(sStatment* statment)
+{
+    sStatment* self = MALLOC(sizeof(sStatment));
+
+    self->mCommands = VECTOR_NEW(30);
+    int i;
+    for(i=0; i<vector_size(statment->mCommands); i++) {
+        vector_add(self->mCommands, sCommand_new2(vector_item(statment->mCommands, i)));
+    }
+    self->mTerminated = statment->mTerminated;
+    self->mFName = STRING_NEW(string_c_str(statment->mFName));
+    self->mLine = statment->mLine;
+    self->mTitle = STRING_NEW(string_c_str(statment->mTitle));
+    if(statment->mNotEvaled) {
+        self->mNotEvaled = STRING_NEW(string_c_str(statment->mNotEvaled));
+    }
+    else {
+        self->mNotEvaled = NULL;
+    }
+    self->mBackground = statment->mBackground;
+    self->mRCodeReverse = statment->mRCodeReverse;
+
+    self->mGlobalPipeIn = statment->mGlobalPipeIn;
+    self->mGlobalPipeOut = statment->mGlobalPipeOut;
+
+    return self;
+}
+
+
 void sStatment_view(sStatment* self)
 {
     printf("command number %d\n", vector_size(self->mCommands));
@@ -2242,6 +2415,18 @@ sStatments* sStatments_new()
 
 #endif
 
+sStatments* sStatments_new2(sStatments* statments)
+{
+    sStatments* self = MALLOC(sizeof(sStatments));
+
+    self->mStatments = VECTOR_NEW(30);
+    int i;
+    for(i=0; i < vector_size(statments->mStatments); i++) {
+        vector_add(self->mStatments, sStatment_new2(vector_item(statments->mStatments, i)));
+    }
+
+    return self;
+}
 void sStatments_view(sStatments* self)
 {
 
@@ -2313,6 +2498,23 @@ sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_na
     return self;
 }
 
+sFunction* sFunction_new2(sFunction* func)
+{
+
+    sFunction* self = MALLOC(sizeof(sFunction));
+    self->name = STRING_NEW(string_c_str(func->name));
+    self->arg_name = STRING_NEW(string_c_str(func->arg_name));
+    
+    if(func->statments) {
+        self->statments = sStatments_new2(func->statments);
+    }
+    else {
+        self->statments = NULL;
+    }
+
+    return self;
+}
+
 void sFunction_delete(sFunction* self)
 {
     string_delete(self->name);
@@ -2728,13 +2930,12 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gFuncs = HASH_NEW(50);
 
     gJobs = VECTOR_NEW(50);
-    gSigChldRecieved = LIST_NEW();
 
     gGlobals = HASH_NEW(100);
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "1.3.5", 1);
+    setenv("SAPHIRE_VERSION", "1.3.7", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2772,14 +2973,6 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gDirStack = VECTOR_NEW(10);
     gErrMsg = STRING_NEW("");
 
-#if defined(__LINUX__) || defined(__DARWIN__) || defined(__FREEBSD__)
-    gMsgId = msgget((key_t)1234, 0666 | IPC_CREAT);
-    if(gMsgId < 0) {
-        fprintf(stderr, "msgget failed\n");
-        exit(EXIT_FAILURE);
-    }
-#endif
-
 
 //    saphire_rehash();
 
@@ -2810,7 +3003,6 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     else 
         xstrncpy(gTmpDir, "/tmp", PATH_MAX);
 
-    gMatchRestart = HASH_NEW(1000);
     gStackTraceFName = VECTOR_NEW(30);
     gStackTraceLineNum = VECTOR_NEW(30);
 
@@ -2827,8 +3019,6 @@ void saphire_final()
     vector_delete(gStackTraceFName);
     vector_delete(gStackTraceLineNum);
 
-    hash_delete(gMatchRestart);
-
     int len = vector_size(gGlobalPipes);
     for(i=0; i<len; i++) {
         string_delete(vector_item(gGlobalPipes, i));
@@ -2906,8 +3096,6 @@ void saphire_final()
     }
     vector_delete(gJobs);
 
-    list_delete(gSigChldRecieved);
-
     it = hash_loop_begin(gReadBuffers);
     while(it) {
         sRFd* fd = (sRFd*)hash_loop_item(it);
@@ -2969,12 +3157,6 @@ void saphire_final()
     }
     vector_delete(gStrayCProgs);
 
-#if defined(__LINUX__) || defined(__DARWIN__) || defined(__FREEBSD__)
-    if(msgctl(gMsgId, IPC_RMID, 0) == -1) {
-        fprintf(stderr, "msgctl(IPC_RMID) failed\n");
-    }
-#endif
-
     it = hash_loop_begin(gInnerCommands);
     while(it != NULL) {
         sInnerCommand_delete(hash_loop_item(it));
@@ -3779,6 +3961,20 @@ sBlock* sBlock_new(sStatments* statments, vector_obj* args)
     return self;
 }
 
+sBlock* sBlock_new2(sBlock* block)
+{
+    sBlock* self = MALLOC(sizeof(sBlock));
+
+    self->mStatments = sStatments_new2(block->mStatments);
+    self->mArgs = VECTOR_NEW(10);
+    int i;
+    for(i=0; i<vector_size(block->mArgs); i++) {
+        vector_add(self->mArgs, STRING_NEW(string_c_str(vector_item(block->mArgs, i))));
+    }
+
+    return self;
+}
+
 void sBlock_delete(sBlock* self)
 {
     sStatments_delete(self->mStatments);
